@@ -4,7 +4,6 @@ from copy import deepcopy
 from datetime import datetime
 from math import ceil
 from random import choice, randint, uniform
-import warnings
 
 from .consts import MAX_LABEL_COST, SMALL_DIVISOR
 from .path import find_path_for_agents, find_shortest_path, \
@@ -75,7 +74,7 @@ class Link:
                  lanes=1,
                  link_type=1,
                  free_speed=60,
-                 capacity=49500,
+                 capacity=1999,
                  allowed_uses='all',
                  geometry='',
                  demand_period_size=1):
@@ -99,8 +98,6 @@ class Link:
         self.link_capacity = capacity * lanes
         self.allowed_uses = allowed_uses
         self.geometry = geometry
-        self.cost = self.fftt
-        self.flow_volume = 0
         # add for CG
         self.demand_period_size = demand_period_size
         self.toll = 0
@@ -175,8 +172,8 @@ class Link:
     def increase_period_flow_vol(self, tau, fv):
         self.flow_vol_by_period[tau] += fv
 
-    def calculate_td_vdf(self, demand_periods=None, iter_num=None):
-        if demand_periods is None or iter_num is None:
+    def calculate_td_vdf(self, demand_periods=None, iter_num=-1):
+        if demand_periods is None:
             for tau in range(self.demand_period_size):
                 self.travel_time_by_period[tau] = (
                     self.vdfperiods[tau].run_bpr(self.flow_vol_by_period[tau])
@@ -257,7 +254,7 @@ class Agent:
     def get_dp_id(self):
         return self.dp_id
 
-    def update_dep_time(self, tt):
+    def update_dep_interval(self, tt):
         self.link_dep_interval[self.curr_link_pos] = (
             self.link_arr_interval[self.curr_link_pos] + tt
         )
@@ -272,13 +269,13 @@ class Agent:
         pos = self.curr_link_pos - 1
         return self.link_path[pos]
 
-    def set_dep_time(self, i):
+    def set_dep_interval(self, i):
         self.link_dep_interval[self.curr_link_pos] = i
 
-    def set_arr_time(self, i, increment=0):
+    def set_arr_interval(self, i, increment=0):
         self.link_arr_interval[self.curr_link_pos-increment] = i
 
-    def get_arr_time(self):
+    def get_arr_interval(self):
         return self.link_arr_interval[self.curr_link_pos]
 
     def increment_link_pos(self):
@@ -423,7 +420,7 @@ class Network:
         # initialize from_node_no_array, to_node_no_array, and link_cost_array
         from_node_no_array = [link.from_node_seq_no for link in self.links]
         to_node_no_array = [link.to_node_seq_no for link in self.links]
-        link_cost_array = [link.cost for link in self.links]
+        link_cost_array = [link.fftt for link in self.links]
 
         # initialize others
         queue_next = [0] * node_size
@@ -821,21 +818,24 @@ class Column:
     def set_gradient_cost(self, c):
         self.gradient_cost = c
 
-    def set_gradient_cost_abs_diff(self, ad):
-        self.gradient_cost_abs_diff = ad
-
-    def set_gradient_cost_rel_diff(self, rd):
-        self.gradient_cost_rel_diff = rd
-
-    def increase_toll(self, t):
-        self.toll += t
-
     def increase_volume(self, v):
         self.vol += v
 
     def set_geometry(self, g):
         self.geo = g
 
+    def update_gradient_cost_diffs(self, least_gc):
+        self.gradient_cost_abs_diff = self.gradient_cost - least_gc
+        self.gradient_cost_rel_diff = (
+            self.gradient_cost_abs_diff / max(SMALL_DIVISOR, least_gc)
+        )
+
+    def get_gap(self):
+        return self.gradient_cost_abs_diff * self.vol
+
+    def get_sys_travel_time(self):
+        return self.gradient_cost * self.vol
+
 
 class ColumnVec:
     """ column pool for (at, dp, oz, dz)
@@ -1075,7 +1075,7 @@ class SPNetwork(Network):
         link_preds = [-1] * base.node_size
         node_lables = [MAX_LABEL_COST] * base.node_size
         queue_next = [0] * base.node_size
-        link_cost_array = [link.cost for link in base.links]
+        link_cost_array = [link.fftt for link in base.links]
 
         int_arr_node = ctypes.c_int * base.node_size
         double_arr_node = ctypes.c_double * base.node_size
@@ -1347,7 +1347,7 @@ class Assignment:
         self.network = None
         self.spnetworks = []
         self.accessnetwork = None
-        self.memory_blocks = 4
+        self.memory_blocks = 1
         self.map_atstr_id = {}
         self.map_dpstr_id = {}
         self.map_name_atstr = {}
@@ -1610,11 +1610,11 @@ class Assignment:
         # output the entire node set directly without the following check?
         nodes = []
         for node in self.accessnetwork.get_nodes():
-            node_no = node.get_node_no()
             # do not include the source node itself
             if node.get_node_id() == source_node_id:
                 continue
 
+            node_no = node.get_node_no()
             if self.accessnetwork.get_node_label_cost(node_no) <= time_budget:
                 nodes.append(node.get_node_id())
 
